@@ -18,7 +18,7 @@ use File::Path ();
 use Cwd ();
 use Config;
 
-our $VERSION = '0.292';
+our $VERSION = '0.293';
 
 use constant WIN32 => $^O eq 'MSWin32';
 
@@ -32,7 +32,6 @@ sub new {
         local_lib => "local",
         cpanmetadb => "http://cpanmetadb.plackperl.org/v1.0/",
         mirror => ["http://www.cpan.org/", "http://backpan.perl.org/"],
-        target_perl => $],
         %option
     }, $class;
 }
@@ -71,6 +70,7 @@ sub parse_options {
     }
     $self->{color} = 1 if !defined $self->{color} && -t STDOUT;
     if ($target_perl) {
+        die "--target-perl option conflicts with --global option\n" if $self->{global};
         # 5.8 is interpreted as 5.800, fix it
         $target_perl = "v$target_perl" if $target_perl =~ /^5\.[1-9]\d*$/;
         $self->{target_perl} = App::cpm::version->parse($target_perl)->numify;
@@ -90,27 +90,25 @@ sub parse_options {
     $self->{argv} = \@ARGV;
 }
 
-sub _core_inc {
+sub _inc {
     my $self = shift;
-    (
-        (!$self->{exclude_vendor} ? grep {$_} @Config{qw(vendorarch vendorlibexp)} : ()),
-        @Config{qw(archlibexp privlibexp)},
-    );
-}
-
-sub _user_inc {
-    my $self = shift;
-    if ($self->{global}) {
-        my %core = map { $_ => 1 } $self->_core_inc;
-        return grep { !$core{$_} } @INC;
-    }
+    return \@INC if $self->{global};
 
     my $base = $self->{local_lib};
     require local::lib;
-    (
+    my @local_lib = (
         local::lib->resolve_path(local::lib->install_base_arch_path($base)),
         local::lib->resolve_path(local::lib->install_base_perl_path($base)),
     );
+    my @core = (
+        (!$self->{exclude_vendor} ? grep {$_} @Config{qw(vendorarch vendorlibexp)} : ()),
+        @Config{qw(archlibexp privlibexp)},
+    );
+    if ($self->{target_perl}) {
+        return [@local_lib];
+    } else {
+        return [@local_lib, @core];
+    }
 }
 
 sub maybe_abs {
@@ -180,9 +178,8 @@ sub cmd_install {
 
     my $master = App::cpm::Master->new(
         logger => $logger,
-        core_inc => [$self->_core_inc],
-        user_inc => [$self->_user_inc],
-        target_perl => $self->{target_perl},
+        inc    => $self->_inc,
+        (exists $self->{target_perl} ? (target_perl => $self->{target_perl}) : ()),
     );
     $self->register_initial_job($master);
 
@@ -273,18 +270,18 @@ sub register_initial_job {
             my $dist = App::cpm::Distribution->new(source => "http", uri => $arg, provides => []);
             $master->add_distribution($dist);
         } else {
-            my ($package, $version, $dev);
+            my ($package, $version_range, $dev);
             # copy from Menlo
             # Plack@1.2 -> Plack~"==1.2"
             $arg =~ s/^([A-Za-z0-9_:]+)@([v\d\._]+)$/$1~== $2/;
             # support Plack~1.20, DBI~"> 1.0, <= 2.0"
             if ($arg =~ /\~[v\d\._,\!<>= ]+$/) {
-                ($package, $version) = split '~', $arg, 2;
+                ($package, $version_range) = split '~', $arg, 2;
             } else {
                 $arg =~ s/[~@]dev$// and $dev++;
                 $package = $arg;
             }
-            push @package, {package => $package, version => $version || 0, dev => $dev};
+            push @package, {package => $package, version_range => $version_range || 0, dev => $dev};
         }
     }
 
@@ -297,7 +294,7 @@ sub register_initial_job {
             exit 0;
         } elsif (!defined $is_satisfied) {
             my ($req) = grep { $_->{package} eq "perl" } @$requirements;
-            die sprintf "%s requires perl %s\n", $self->{cpanfile}, $req->{version};
+            die sprintf "%s requires perl %s\n", $self->{cpanfile}, $req->{version_range};
         } else {
             @package = @need_resolve;
         }
@@ -307,7 +304,7 @@ sub register_initial_job {
         $master->add_job(
             type => "resolve",
             package => $p->{package},
-            version => $p->{version} || 0,
+            version_range => $p->{version_range} || 0,
             dev => $p->{dev},
         );
     }
@@ -339,7 +336,7 @@ sub load_cpanfile {
             );
         } else {
             push @package, {
-                package => $package, version => $hash->{$package}, dev => $option->{dev},
+                package => $package, version_range => $hash->{$package}, dev => $option->{dev},
             };
         }
     }
